@@ -12,7 +12,7 @@ import json
 import os
 import platform
 import sys
-import uuid
+from uuid import uuid4
 import psutil
 import pytz
 
@@ -108,7 +108,7 @@ class Provenance(metaclass=Singleton):
 
         self._main_activity_uuid = self.start_activity("main session")
 
-        atexit.register(self._export, self.outfile)
+        atexit.register(self._export)
 
     @property
     def outfile(self):
@@ -151,11 +151,13 @@ class Provenance(metaclass=Singleton):
         """Record configuration parameters (e.g. of the pipeline)"""
         self.current_activity.record_configuration(configuration)
 
-    def record_input(self, url, comment=""):
-        self.current_activity.record_input(url, comment)
+    def record_input(self, url, uuid=None, comment=""):
+        self.current_activity.record_input(url, uuid, comment)
 
-    def record_output(self, url, comment=""):
-        self.current_activity.record_output(url, comment)
+    def record_output(self, url, uuid=None, comment=""):
+        if uuid is None:
+            uuid = str(uuid4())
+        self.current_activity.record_output(url, uuid, comment)
 
     @property
     def current_activity(self):
@@ -190,33 +192,33 @@ class Provenance(metaclass=Singleton):
 
         return json.dumps(self.provenance, default=fallback, **kwargs)
 
-    def _export(self, outfile):
+    def _export(self):
         """Writes the provenance information into outfile
 
         This function is called automatically upon exit, no manual call is required.
         """
-        if outfile is None:
+        if self.outfile is None:
             return
         try:
             self.finish_activity(self._main_activity_uuid)
         except ValueError:
             log.warning("Could not finish the main session.")
-        print("Provenance information has been written to '{}'".format(outfile))
-        with open(outfile, "w") as fobj:
+        print("Provenance information has been written to '{}'".format(self.outfile))
+        with open(self.outfile, "w") as fobj:
             fobj.write(self.as_json(indent=2))
 
     def reset(self):
         log.info("Resetting provenance")
-        atexit.unregister(self._export)
         self._activities = []
         self._backlog = []
+        self.outfile = None
 
 
 class _Activity:
     def __init__(self, name):
         self.name = name
         self._data = dict(
-            uuid=str(uuid.uuid4()),
+            uuid=str(uuid4()),
             name=name,
             parent_activity=None,
             child_activities=[],
@@ -238,11 +240,11 @@ class _Activity:
         """Records or updates configuration"""
         self._data["configuration"].update(configuration)
 
-    def record_input(self, url, comment):
-        self._data["input"].append(dict(url=url, comment=comment))
+    def record_input(self, url, uuid, comment):
+        self._data["input"].append(dict(url=url, uuid=uuid, comment=comment))
 
-    def record_output(self, url, comment):
-        self._data["output"].append(dict(url=url, comment=comment))
+    def record_output(self, url, uuid, comment):
+        self._data["output"].append(dict(url=url, uuid=uuid, comment=comment))
 
     def finish(self, status):
         self._data["stop"] = system_state()
